import click

from textwrap import TextWrapper


def echo_dict(data,
              no_color,
              key_color='green',
              spaces=None,
              value_color='blue'):
    """
    Echoes a dictionary pretty-print style to terminal.

    :param data: The dictionary of key:vals to print.
    :param no_color: If true prints formatted but with no ascii color.
    :param key_color: Color for dictionary keys.
    :param spaces: Number of spaces to offset values. Max length of keys.
    :param value_color: Color for values.
    """
    if not spaces:
        spaces = get_max_key(data)

    for key, value in data.items():
        title = '{spaces}{key}:  '.format(
            spaces=' ' * (spaces - len(key)),
            key=key
        )
        wrapper = TextWrapper(
            width=(82 - spaces),
            subsequent_indent=' ' * (spaces + 3)
        )

        click.echo(
            ''.join([
                style_string(title, no_color, fg=key_color),
                wrapper.fill(style_string(value, no_color, fg=value_color))
            ])
        )


def echo_style(message, no_color, fg='yellow'):
    """
    Echo string with style if no_color is False.

    :param message: String to echo.
    :param no_color: If True echo without style.
    :param fg: String style color.
    """
    if no_color:
        click.echo(message)
    else:
        click.secho(message, fg=fg)


def get_max_key(data):
    """
    Get the max key length from dictionary.

    :param data: Dictionary of key values.
    :return: Max length of dictionary key as int.
    """
    return max(map(len, data))


def style_string(message, no_color, fg='yellow'):
    """
    Add color style to string if no_color is False.

    :param message: String to style.
    :param no_color: If True return string without style.
    :param fg: The color for the string style.
    :return: Return string conditionally style.
    """
    if no_color:
        return message
    else:
        return click.style(message, fg=fg)



# -*- coding: utf-8 -*-
#
# wxcast: A Python API and cli to collect weather information.
#
# Copyright (C) 2017 Sean Marlow
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import click

from wxcast import api
from wxcast import utils


def print_license(ctx, param, value):
    """
    Eager option to print license information and exit.
    """
    if not value or ctx.resilient_parsing:
        return

    click.echo(
        'wxcast Copyright (C) 2017 sean Marlow. (GPL-3.0+)\n\n'
        'This program comes with ABSOLUTELY NO WARRANTY.\n'
        'This is free software, and you are welcome to redistribute it'
        ' under certain conditions. See LICENSE for more information.'
    )
    ctx.exit()


@click.group()
@click.version_option()
@click.option(
    '--license',
    expose_value=False,
    is_eager=True,
    is_flag=True,
    callback=print_license,
    help='Display license information and exit.'
)
def main():
    """
    Retrieve the latest weather information right in your terminal.

    Data provided by NWS and AVWX.

    NWS: https://forecast-v3.weather.gov/documentation \n
    AVWX: https://avwx.rest/
    """
    pass


@click.command()
@click.option(
    '--no-color',
    is_flag=True,
    help='Remove ANSI color and styling from output.'
)
@click.argument('location')
def forecast(location, no_color):
    """
    Retrieve current 7 day forecast for given location.

    Location can be a city, address or zip/postal code.

    Example: wxcast forecast -L denver

    :param location: Location string to get forecast for.
    :param no_color: If True do not style string output.
    """
    try:
        response = api.get_seven_day_forecast(location)
    except Exception as e:
        utils.echo_style(str(e), no_color, fg='red')
    else:
        data = {d['name']: d['detailedForecast'] for d in response}
        utils.echo_dict(data, no_color)


@click.command()
@click.option(
    '-d', '--decoded',
    is_flag=True,
    help='Decode raw metar to string format.'
)
@click.option(
    '--no-color',
    is_flag=True,
    help='Remove ANSI color and styling from output.'
)
@click.argument('icao')
def metar(decoded, no_color, icao):
    """
    Retrieve the latest METAR given an airport ICAO code.

    Example: wxcast metar -d KSLC

    :param decoded: Flag to decode the METAR output.
    :param no_color: If True do not style string output.
    :param icao: The airport ICAO code to retrieve METAR for.
    """
    try:
        response = api.get_metar(icao, decoded)
    except Exception as e:
        utils.echo_style(str(e), no_color, fg='red')
    else:
        if decoded:
            click.echo(
                ''.join([
                    utils.style_string(
                        'At ', no_color, fg='green'
                    ),
                    utils.style_string(
                        response['header']['time'], no_color, fg='blue'
                    ),
                    utils.style_string(
                        ' the conditions for ', no_color, fg='green'
                    ),
                    utils.style_string(
                        response['header']['icao'], no_color, fg='blue'
                    ),
                    utils.style_string(
                        ' are ', no_color, fg='green'
                    ),
                    utils.style_string(
                        response['header']['fr'], no_color, fg='blue'
                    ),
                    '\n'
                ])
            )

            spaces = max([
                utils.get_max_key(response['data']),
                utils.get_max_key(response['location'])
            ])

            utils.echo_dict(response['data'], no_color, spaces=spaces)
            click.echo('')

            try:
                # Try to convert elevation to ft and meters.
                response['location']['Elevation'] = '{}ft ({}m)'.format(
                    int(float(response['location']['Elevation']) * 3.28084),
                    response['location']['Elevation']
                )
            except:
                pass

            utils.echo_dict(response['location'], no_color, spaces=spaces)

        else:
            utils.echo_style(response, no_color, fg='blue')


@click.command()
@click.option(
    '--no-color',
    is_flag=True,
    help='Remove ANSI color and styling from output.'
)
@click.argument('wfo')
def products(no_color, wfo):
    """
    Retrieve the available text products for a given wfo.

    Example: wxcast products slc

    :param no_color: If True do not style string output.
    :param wfo: The weather forecast office abbreviation (BOU).
    """
    try:
        response = api.get_wfo_products(wfo)
    except Exception as e:
        utils.echo_style(str(e), no_color, fg='red')
    else:
        utils.echo_dict(response, no_color)


@click.command()
@click.option(
    '--no-color',
    is_flag=True,
    help='Remove ANSI color and styling from output.'
)
@click.argument('wfo')
@click.argument('product')
def text(no_color, wfo, product):
    """
    Retrieve the NWS text product.

    Example: wxcast text slc afd

    :param no_color: If True do not style string output.
    :param wfo: The weather forecast office abbreviation (BOU).
    :param product: The text product to retrieve.
    """
    try:
        response = api.get_nws_product(wfo, product)
    except Exception as e:
        utils.echo_style(str(e), no_color, fg='red')
    else:
        click.echo_via_pager(response)


main.add_command(metar)
main.add_command(text)
main.add_command(products)
main.add_command(forecast)



# -*- coding: utf-8 -*-
#
# wxcast: A Python API and cli to collect weather information.
#
# Copyright (C) 2017 Sean Marlow
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import requests

from geopy.geocoders import Nominatim

from wxcast.constants import HEADERS, NWS_API
from wxcast.exceptions import WxcastException


def get_metar(icao, decoded=False):
    """
    Retrieve METAR for ICAO.

    :param icao: Airport code.
    :param decoded: Flag to decode the METAR.
    :return: Returns raw METAR string or dictionary with decoded info.
    """
    try:
        site = f'http://avwx.rest/api/metar/{icao}?options=info,translate'
        data = requests.get(site).json()

    except requests.exceptions.ConnectionError:
        raise WxcastException(
            'Connection could not be established with the avwx rest api.'
        )
    except Exception as e:
        raise WxcastException(f'An error has occurred: {str(e)}')

    if 'Error' in data:
        raise WxcastException(data['Error'])

    if decoded:
        header = {
            'time': data['Time'],
            'icao': data['Station'],
            'fr': data['Flight-Rules']
        }
        output = {
            'data': data['Translations'],
            'header': header,
            'location': data['Info']
        }

        return output

    # else return raw metar
    return data['Raw-Report']


def get_nws_product(wfo, product):
    """
    Returns text from product for given WFO.

    :param wfo: Weather forecast office abbreviation code.
    :param product: The text product to return.
    :return: Product text value as string.
    """
    site = f'{NWS_API}/products/types/' \
           f'{product.upper()}/locations/{wfo.upper()}'

    try:
        response = requests.get(site, headers=HEADERS)

        if response.status_code == 404:
            raise Exception(
                f'No wx data found attempting to retrieve '
                f'{product} issued by {wfo}.'
            )

        response = response.json()
        if not response.get('features', None):
            raise Exception(
                f'No wx data found attempting to retrieve '
                f'{product} issued by {wfo}.'
            )

        response = requests.get(
            response['features'][0]['@id'],
            headers=HEADERS
        ).json()

    except requests.exceptions.ConnectionError as e:
        raise WxcastException(
            f'Connection could not be established with the NWS website: '
            f'{str(e)}'
        )
    except Exception as e:
        raise WxcastException(str(e))

    return response['productText']


def get_seven_day_forecast(location):
    """
    Retrieve seven day forecast for the given location.

    :param location: String value of location (address, name, zip, etc).
    :return: A dictionary with forecast split in periods.
    """
    geolocator = Nominatim()
    geolocation = geolocator.geocode(location)
    latlong = f'{geolocation.latitude},{geolocation.longitude}'

    try:
        data = requests.get(
            f'{NWS_API}/points/{latlong}/forecast',
            headers=HEADERS
        ).json()
    except requests.exceptions.ConnectionError:
        raise WxcastException(
            'Connection could not be established with the nws rest api.'
        )
    except Exception as e:
        raise WxcastException(f'An error has occurred: {str(e)}')

    if 'properties' not in data:
        raise WxcastException(
            f'No forecast found for location: {location} '
            f'coordinates: {latlong}'
        )

    return data['properties']['periods']


def get_wfo_products(wfo):
    """
    Get a list of the text products available for the given WFO.

    :param wfo: The weather forecast office to retrieve product list for.
    :return: Return dictionary of text products {code: name}.
    """
    try:
        site = f'{NWS_API}/products/locations/{wfo.upper()}/types'
        data = requests.get(site, headers=HEADERS).json()

    except requests.exceptions.ConnectionError as e:
        raise WxcastException(
            f'Connection could not be established with the avwx rest api: '
            f'{str(e)}'
        )
    except Exception as e:
        raise WxcastException(f'An error has occurred: {str(e)}')

    response = {d['productCode']: d['productName'] for d in data['features']}
    return response
